import { Between, EntityRepository, FindOperator, getConnection, getManager, Like, Not, Repository } from "typeorm";
import CryptoUtils from "../../utils/crypto.utils";
import { CreateUserDto } from "./dto/create-user.dto";
import { UpdateUserDto } from "./dto/update-user.dto";
import { User } from "./entities/user.entity";
import { UserStatusType } from "src/enum-type/user-status.enum";

@EntityRepository(User)
export class UsersRepository extends Repository<User> {
    async createEntity(dto: CreateUserDto): Promise<User | Error> {
        try {
            const { firstName, lastName, email, phone, password: pass } = dto;

            let userType = "";
            let status: any;
            let role = 3;
            if (dto.userType && dto.userType === "admin") {
                userType = dto.userType;
                role = 2;
                status = "Approved" as UserStatusType
            } else {
                userType = "customer";
                status = "Pending" as UserStatusType
            }

            const entity = this.create({
                firstName,
                lastName,
                email,
                phone,
                password: await CryptoUtils.encrypt(pass),
                role,
                userType,
                status
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateUserDto): Promise<User | Error> {
        try {
            const user = await this.findOne(id);
            await this.update(id, {
                firstName: dto.firstName ? dto.firstName : user.firstName,
                lastName: dto.lastName ? dto.lastName : user.lastName,
                phone: dto.phone ? dto.phone : user.phone,
                image: dto.image ? dto.image : user.image,
                email: dto.email ? dto.email : user.email,
                role: dto.role ? dto.role : user.role,
                userType: dto.userType ? dto.userType : user.userType,
                companyName: dto.companyName ? dto.companyName : user.companyName,
                streetAddress: dto.streetAddress ? dto.streetAddress : user.streetAddress,
                city: dto.city ? dto.city : user.city,
                state: dto.state ? dto.state : user.state,
                postalCode: dto.postalCode ? dto.postalCode : user.postalCode,
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAllEntities(): Promise<User[] | Error> {
        try {
            return await this.find();
        } catch (e) {
            return e;
        }
    }

    async findByQuery(query: any): Promise<User[] | Error> {
        try {
            query = { ...query, isDeleted: false };
            return await this.find({
                order: { createdAt: -1 },
                where: query
            });
        } catch (e) {
            return e;
        }
    }

    async findAllCommonTransactionUser(userId: string): Promise<User[] | Error> {
        try {
            const query =
                `SELECT DISTINCTROW u.id, u.firstName, u.lastName, u.email, u.phone, u.image, u.role, u.userType
            FROM party p1
            INNER JOIN party p2 ON p2.transactionId = p1.transactionId
            INNER JOIN users u ON u.id = p1.userId
            WHERE p2.userId = '${userId}'
            AND p1.userId <> '${userId}'
            AND u.isDeleted = false;`;

            return await this.manager.query(query);
        } catch (error) {
            return error;
        }
    }

    async findCurrentTransactionUser(id: string, userType?: string, search?: string): Promise<User[] | Error> {
        try {
            const where = [];
            if (search && search != '') {
                where.push({ userType, firstName: Like(`%${search}%`), id: Not(id) });
                where.push({ userType, lastName: Like(`%${search}%`), id: Not(id) });
            } else {
                where.push({ userType, id: Not(id) });
            }
            return await this.find({
                order: { createdAt: -1 },
                where
            });
        } catch (e) {
            return e;
        }
    }

    async findUserForDropDown(searchKeyword?: any): Promise<User[] | Error> {
        try {
            type FindOption = {
                name?: FindOperator<string>;
            };

            const findOptions: FindOption = {};

            if (searchKeyword) findOptions.name = Like("%" + searchKeyword + "%");

            // return await this.find({
            //     select :["id","name"],
            //     where: {...findOptions, isDeleted: false, isApproved: true}
            // });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<User | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findOneUserDetailsEntity(request: any): Promise<User | Error> {
        try {
            return await this.findOne({
                where: {
                    id: request.user.id,
                    role: request.user.role,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findByEmailAddress(email: string): Promise<User | Error> {
        try {
            return await this.findOne({
                where: { email: email, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async findByEmailAddressAndActive(
        email: string,
        role: number
    ): Promise<User | Error> {
        try {
            return await this.findOne({
                where: {
                    email: email,
                    role: role,
                    status: "Approved" as UserStatusType,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(User)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return false;
        }
    }

    async updateUserPassword(
        id: string,
        password: string
    ): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(User)
                .set({ password: password })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return false;
        }
    }

    async countCustomer(status: number): Promise<any | Error> {
        try {
            return await this.count({
                where: { isDeleted: false, role: status }
            });
        } catch (e) {
            return e;
        }
    }

    async monthlyCustomer(): Promise<any | Error> {
        try {
            let monthWiseCount: any = [];
            const date = new Date();
            for (let i = 0; i < 12; i++) {
                const fyear = new Date(date.getFullYear(), i, 1).toLocaleString(
                    "default",
                    { year: "numeric" }
                );
                const fmonth = new Date(date.getFullYear(), i, 1).toLocaleString(
                    "default",
                    { month: "2-digit" }
                );
                const fday = new Date(date.getFullYear(), i, 1).toLocaleString(
                    "default",
                    { day: "2-digit" }
                );

                // Generate yyyy-mm-dd date string
                const firstDayFormattedDate = fyear + "-" + fmonth + "-" + fday;

                const lyear = new Date(date.getFullYear(), i + 1, 0).toLocaleString(
                    "default",
                    { year: "numeric" }
                );
                const lmonth = new Date(date.getFullYear(), i + 1, 0).toLocaleString(
                    "default",
                    { month: "2-digit" }
                );
                const lday = new Date(date.getFullYear(), i + 1, 0).toLocaleString(
                    "default",
                    { day: "2-digit" }
                );

                // Generate yyyy-mm-dd date string
                const lastDayFormattedDate = lyear + "-" + lmonth + "-" + lday;

                console.log("First Day: ", firstDayFormattedDate);
                console.log("Last Day: ", lastDayFormattedDate);

                const start = new Date(firstDayFormattedDate);
                start.setUTCHours(0, 0, 0, 0);
                const end = new Date(lastDayFormattedDate);
                end.setUTCHours(23, 59, 59, 999);
                const data = await this.count({
                    where: {
                        isDeleted: false,
                        createdAt: Between(
                            new Date(start).toISOString(),
                            new Date(end).toISOString()
                        )
                    }
                });
                monthWiseCount.push(data ? data : 0);
            }
            return monthWiseCount;
        } catch (e) {
            return e;
        }
    }

    async callcheckSchedulePartyInvitesSP(userId: string): Promise<any> {
        const sql = `CALL checkSchedulePartyInvites(?);`;
        const entityManager = getManager();
        const data = await entityManager.query(sql, [userId]);

        return data;
    }
}
